/*
  File Name     : ft602_uvc_io.v
  Department    : IC Design, FTDI SGP
  Author        : Do Ngoc Duong
  History       : 25 August 2016 - Modify from fifo_mst_io.v of Kwok Wai 
                  13 December 2016 - Add I2C  

  Description   : This module control the input, output of FPGA  
*/
module ft602_uvc_io (
  //GPIO Control Signals
  input wire HRST_N,
  input wire SRST_N,
  //
  input wire MLTCN, 
  // FIFO Slave interface 
  input wire CLK,
  inout wire [31:0] DATA,
  output wire [3:0] BE,
  input wire RXF_N,   	// ACK_N
  input wire TXE_N,
  output wire WR_N,    // REQ_N
  output wire SIWU_N,
  output wire RD_N,
  output wire OE_N,
  //
  inout wire I2C_SCL,
  inout wire I2C_SDA,
  output wire IRQ,
  // From chip internal 
  input wire [31:0] tp_data,
  input wire [3:0]  tp_be,
  input wire tp_dt_oe_n,
  input wire tp_be_oe_n,
  input wire tp_siwu_n,
  input wire tp_wr_n,    
  input wire tp_rd_n,
  input wire tp_oe_n,
  //
  input wire tp_i2c_oe,
  input wire tp_i2c_sdo,
  // To chip internal 
  output wire tc_i2c_scl,
  output wire tc_i2c_sdi, 
  //
  input  tp_int_ena,
  input  tp_int_irq,
  // 
  output wire [31:0] tc_data,
  output wire [3:0]  tc_be,
  output wire tc_rst_n,
  output wire tc_clk,
  output wire tc_txe_n,
  output wire tc_rxf_n,
  //
  output wire tc_mltcn
);
  // 
  `ifdef FTDI_SIM 
  localparam INP_DLY  = 2ns;
  localparam OUT_DLY  = 2.5ns;
  localparam CLK_DLY  = 4ns;
  `else 
  localparam INP_DLY  = 0;
  localparam OUT_DLY  = 0;
  localparam CLK_DLY  = 0;
  `endif 
  // Clock 
  assign #CLK_DLY tc_clk 	= CLK; 
  // data
  assign #INP_DLY tc_data[7:0]  = DATA[7:0];
  assign #INP_DLY tc_data[15:8] = DATA[15:8];
  assign #INP_DLY tc_data[31:16]= DATA[31:16];
  assign #INP_DLY tc_be  	= BE;
  // 
  assign #OUT_DLY DATA[7:0] 	= ~tp_be_oe_n ? tp_data[7:0] 	: 8'bzzzzzzzz;
  assign #OUT_DLY DATA[15:8] 	= ~tp_dt_oe_n ? tp_data[15:8] 	: 8'bzzzzzzzz;
  assign #OUT_DLY DATA[31:16] 	= ~tp_be_oe_n ? tp_data[31:16] 	: 16'hzzzz;
  // assign #OUT_DLY BE 		= ~tp_be_oe_n ? tp_be 		: 4'bzzzz;
  //
  //assign #OUT_DLY DATA[7:0] 	= tp_data[7:0];
  //assign #OUT_DLY DATA[15:8] 	= tp_data[15:8];
  //assign #OUT_DLY DATA[31:16] = tp_data[31:16];
  assign #OUT_DLY BE 		   = tp_be;
  // control  
  assign #INP_DLY tc_txe_n 	= TXE_N;
  assign #INP_DLY tc_rxf_n 	= RXF_N;
  // 
  assign #OUT_DLY SIWU_N  	= tp_siwu_n;
  assign #OUT_DLY WR_N 		= tp_wr_n;
  assign #OUT_DLY RD_N    	= tp_rd_n;
  assign #OUT_DLY OE_N    	= tp_oe_n;
  //GPIO Control  
  assign tc_rst_n 		= SRST_N & HRST_N; 
  assign tc_mltcn		= MLTCN;
  //I2C 
  assign I2C_SDA    = tp_i2c_oe ? tp_i2c_sdo : 1'bz; 
  assign tc_i2c_scl = I2C_SCL;
  assign tc_i2c_sdi = (!tp_i2c_oe) ? I2C_SDA : 1'b0; 
  //
  assign IRQ = tp_int_ena ? tp_int_irq : 1'bz;  
//
endmodule 
