/*
  File Name     : ft602_uvc_reg.v  
  Department    : IC Design, FTDI SGP
  Author        : Do Ngoc Duong
  History       : 13 Dec. 2016 - Initial Version 

  Description   : This module contains registers  
*/

module ft602_uvc_reg (
  //i2c interface
  input  clk,
  input  rst_n,
  input  reg_wr,
  input  reg_rd,
  input  [7:0]   reg_adr,
  input  [127:0] reg_di,
  output reg          reg_rdy,
  output reg  [127:0] reg_do,
  //Registers interface
  output wakeup,
  output reg [7:0]   mode_select, 
  //
  output wire [15:0]  cn0_control, 
  output wire [31:0]  cn0_v_width, 
  output wire [15:0]  cn0_l_valid, 
  output wire [15:0]  cn0_lsg_leg, 
  output wire [15:0]  cn0_h_blank, 
  output wire [15:0]  cn0_d_valid,
  output wire [15:0]  cn0_dsg_deg,
  //
  output wire [15:0]  cn1_control, 
  output wire [31:0]  cn1_v_width, 
  output wire [15:0]  cn1_l_valid, 
  output wire [15:0]  cn1_lsg_leg, 
  output wire [15:0]  cn1_h_blank, 
  output wire [15:0]  cn1_d_valid,
  output wire [15:0]  cn1_dsg_deg,
  //
  output wire [15:0]  cn2_control, 
  output wire [31:0]  cn2_v_width, 
  output wire [15:0]  cn2_l_valid, 
  output wire [15:0]  cn2_lsg_leg, 
  output wire [15:0]  cn2_h_blank, 
  output wire [15:0]  cn2_d_valid,
  output wire [15:0]  cn2_dsg_deg,
  //
  output wire [15:0]  cn3_control, 
  output wire [31:0]  cn3_v_width, 
  output wire [15:0]  cn3_l_valid, 
  output wire [15:0]  cn3_lsg_leg, 
  output wire [15:0]  cn3_h_blank, 
  output wire [15:0]  cn3_d_valid,
  output wire [15:0]  cn3_dsg_deg,
  //
  output reg [15:0]  ch0_low_mark, 
  output reg [15:0]  ch0_high_mark, 
  output reg [15:0]  ch1_low_mark, 
  output reg [15:0]  ch1_high_mark, 
  output reg [15:0]  ch2_low_mark, 
  output reg [15:0]  ch2_high_mark, 
  output reg [15:0]  ch3_low_mark, 
  output reg [15:0]  ch3_high_mark,
  //
  input  ch0_drop_trig, 
  input  ch1_drop_trig, 
  input  ch2_drop_trig, 
  input  ch3_drop_trig,
  output [3:0] frame_drop,
  output  [7:0]  power_stat,
  // 
  input  [7:0]  DIPSW,
  input  [2:0]  PBUTT,
  //
  output wire int_irq,
  output wire int_ena
);
//*********************************************
reg [2:0] pbutt_p1;
reg [2:0] pbutt_p2;
reg [7:0] dipsw_p1;
reg [7:0] dipsw_p2;
//
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
  begin 
    pbutt_p1 <= 3'h0;
    pbutt_p2 <= 3'h0;
    dipsw_p1 <= 8'h00;	 
    dipsw_p2 <= 8'h00;
  end 
  else 
  begin
    pbutt_p1 <= PBUTT;
	 dipsw_p1 <= DIPSW;
	 pbutt_p2 <= pbutt_p1;
	 dipsw_p2 <= dipsw_p1;	 
  end
end
//
wire [7:0] mem_adr;
wire [127:0] mem_di;
wire [127:0] mem_do;
wire mem_clk;
wire mem_wr;
//
`ifdef ALTERA_FPGA 
spsram_256x128 i_spsram (
	.address  (mem_adr),
	.clock    (mem_clk),
	.data     (mem_di),
	.wren     (mem_wr),
	.q        (mem_do)
	);
`else 
spsram_256x128 i_spsram (
  .clk (mem_clk),
  .we  (mem_wr),
  .a   (mem_adr),
  .d   (mem_di),
  .spo (mem_do)
);
`endif
	
assign mem_clk = clk;
assign mem_wr  = reg_wr && ((reg_adr[7:4] == 4'h4) || (reg_adr[7:4] == 4'h5) || 
                            (reg_adr[7:4] == 4'h7) || (reg_adr[7:4] == 4'h8) || 
                            (reg_adr[7:4] == 4'hA) || (reg_adr[7:4] == 4'hB) || 
                            (reg_adr[7:4] == 4'hD) || (reg_adr[7:4] == 4'hE) );
assign mem_adr = reg_adr;
assign mem_di  = reg_di;
//*********************************************
reg [31:0] ch0_frame_drop; 
reg [31:0] ch1_frame_drop; 
reg [31:0] ch2_frame_drop; 
reg [31:0] ch3_frame_drop; 
reg [71:0] ch0_start_stream;
reg [71:0] ch1_start_stream;
reg [71:0] ch2_start_stream;
reg [71:0] ch3_start_stream;
reg [7:0]  ch0_stop_stream;
reg [7:0]  ch1_stop_stream;
reg [7:0]  ch2_stop_stream;
reg [7:0]  ch3_stop_stream;
reg [7:0]   power_status; 
//
reg [15:0]  ch0_control;
reg [15:0]  ch0_lsg_leg; 
reg [15:0]  ch0_dsg_deg;
reg [15:0]  ch0_h_blank;
// 
reg [15:0]  ch1_control; 
reg [15:0]  ch1_lsg_leg; 
reg [15:0]  ch1_dsg_deg;
reg [15:0]  ch1_h_blank;
//
reg [15:0]  ch2_control; 
reg [15:0]  ch2_lsg_leg; 
reg [15:0]  ch2_dsg_deg;
reg [15:0]  ch2_h_blank;
// 
reg [15:0]  ch3_control; 
reg [15:0]  ch3_lsg_leg; 
reg [15:0]  ch3_dsg_deg;
reg [15:0]  ch3_h_blank;
//
wire enable;
assign enable = 1'b1; //(power_status == 8'h00);
wire cn0_ena;
assign cn0_ena = (!dipsw_p2[0]) & enable;
//
assign cn0_control = {ch0_control[15:5],dipsw_p2[4],ch0_start_stream[66:64],cn0_ena};
assign cn0_v_width = {ch0_start_stream[39:32],ch0_start_stream[47:40],
                      ch0_start_stream[55:48],ch0_start_stream[63:56]};							 
assign cn0_l_valid = {ch0_start_stream[23:16],ch0_start_stream[31:24]};
assign cn0_lsg_leg = ch0_lsg_leg;
assign cn0_h_blank = ch0_h_blank;
assign cn0_d_valid = {ch0_start_stream[7:0],ch0_start_stream[15:8]};
assign cn0_dsg_deg = ch0_dsg_deg;
//
wire cn1_ena;
assign cn1_ena = (!dipsw_p2[1])  & enable;
//
assign cn1_control = {ch1_control[15:5],dipsw_p2[5],ch1_start_stream[66:64],cn1_ena};
assign cn1_v_width = {ch1_start_stream[39:32],ch1_start_stream[47:40],
                      ch1_start_stream[55:48],ch1_start_stream[63:56]};							 
assign cn1_l_valid = {ch1_start_stream[23:16],ch1_start_stream[31:24]};
assign cn1_lsg_leg = ch1_lsg_leg;
assign cn1_h_blank = ch1_h_blank;
assign cn1_d_valid = {ch1_start_stream[7:0],ch1_start_stream[15:8]};
assign cn1_dsg_deg = ch1_dsg_deg;
//
wire cn2_ena;
assign cn2_ena = (!dipsw_p2[2])  & enable; 
assign cn2_control = {ch2_control[15:5],dipsw_p2[6],ch2_start_stream[66:64],cn2_ena};
assign cn2_v_width = {ch2_start_stream[39:32],ch2_start_stream[47:40],
                      ch2_start_stream[55:48],ch2_start_stream[63:56]};							 
assign cn2_l_valid = {ch2_start_stream[23:16],ch2_start_stream[31:24]};
assign cn2_lsg_leg = ch2_lsg_leg;
assign cn2_h_blank = ch2_h_blank;
assign cn2_d_valid = {ch2_start_stream[7:0],ch2_start_stream[15:8]};
assign cn2_dsg_deg = ch2_dsg_deg;
//
wire cn3_ena;
assign cn3_ena = (!dipsw_p2[3]) & enable;
assign cn3_control = {ch3_control[15:5],dipsw_p2[7],ch3_start_stream[66:64],cn3_ena};
assign cn3_v_width = {ch3_start_stream[39:32],ch3_start_stream[47:40],
                      ch3_start_stream[55:48],ch3_start_stream[63:56]};							 
assign cn3_l_valid = {ch3_start_stream[23:16],ch3_start_stream[31:24]};
assign cn3_lsg_leg = ch3_lsg_leg;
assign cn3_h_blank = ch3_h_blank;
assign cn3_d_valid = {ch3_start_stream[7:0],ch3_start_stream[15:8]};
assign cn3_dsg_deg = ch3_dsg_deg;
//
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
    reg_rdy <= 1'b0;
  else 
    reg_rdy <= reg_wr | reg_rd;
end 
//
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
    reg_do <= 'b0;
  else 
    case (reg_adr)
      8'h00 : reg_do <= {{120{1'b0}},mode_select};
      8'h01 : reg_do <= {{120{1'b0}},power_status};
      //
      8'h10 : reg_do <= {{112{1'b0}},ch0_control};
      8'h11 : reg_do <= {{112{1'b0}},ch0_lsg_leg};
      8'h12 : reg_do <= {{112{1'b0}},ch0_dsg_deg};
      8'h13 : reg_do <= {{112{1'b0}},ch0_h_blank};
      //  
      8'h14 : reg_do <= {{112{1'b0}},ch1_control};
      8'h15 : reg_do <= {{112{1'b0}},ch1_lsg_leg};
      8'h16 : reg_do <= {{112{1'b0}},ch1_dsg_deg};
      8'h17 : reg_do <= {{112{1'b0}},ch1_h_blank};
      //
      8'h18 : reg_do <= {{112{1'b0}},ch2_control};
      8'h19 : reg_do <= {{112{1'b0}},ch2_lsg_leg};      
      8'h1A : reg_do <= {{112{1'b0}},ch2_dsg_deg};
      8'h1B : reg_do <= {{112{1'b0}},ch2_h_blank};		
      //  
      8'h1C : reg_do <= {{112{1'b0}},ch3_control};
      8'h1D : reg_do <= {{112{1'b0}},ch3_lsg_leg};
      8'h1E : reg_do <= {{112{1'b0}},ch3_dsg_deg};
      8'h1F : reg_do <= {{112{1'b0}},ch3_h_blank};		
      //
      8'h20 : reg_do <= {{112{1'b0}},ch0_low_mark};
      8'h21 : reg_do <= {{112{1'b0}},ch0_high_mark};
      8'h22 : reg_do <= {{112{1'b0}},ch1_low_mark};
      8'h23 : reg_do <= {{112{1'b0}},ch1_high_mark};
      8'h24 : reg_do <= {{112{1'b0}},ch2_low_mark};
      8'h25 : reg_do <= {{112{1'b0}},ch2_high_mark};
      8'h26 : reg_do <= {{112{1'b0}},ch3_low_mark};
      8'h27 : reg_do <= {{112{1'b0}},ch3_high_mark};
      8'h28 : reg_do <= {{96{1'b0}},ch0_frame_drop};
      8'h29 : reg_do <= {{96{1'b0}},ch1_frame_drop};
      8'h2A : reg_do <= {{96{1'b0}},ch2_frame_drop};
      8'h2B : reg_do <= {{96{1'b0}},ch3_frame_drop};
      //
      8'h60 : reg_do <= {{56{1'b0}},ch0_start_stream};
      8'h61 : reg_do <= {{120{1'b0}},ch0_stop_stream};
      //
      8'h90 : reg_do <= {{56{1'b0}},ch1_start_stream};
      8'h91 : reg_do <= {{120{1'b0}},ch1_stop_stream};
      //
      8'hC0 : reg_do <= {{56{1'b0}},ch2_start_stream};
      8'hC1 : reg_do <= {{120{1'b0}},ch2_stop_stream};
      //
      8'hF0 : reg_do <= {{56{1'b0}},ch3_start_stream};
      8'hF1 : reg_do <= {{120{1'b0}},ch3_stop_stream};				
      default : reg_do <= mem_do; 
    endcase
end 
// 
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
  begin
    mode_select <= 8'h03; 
    power_status<= 8'h00;
    //
    ch0_control <= 16'h3C00; //'b0000_0000;  
    ch0_lsg_leg <= 16'h0804;  
    ch0_dsg_deg <= 16'h0804; 
    ch0_h_blank <= 16'h0010; 
    //
    ch1_control <= 16'h3C20; //8'b0010_0000;  
    ch1_lsg_leg <= 16'h0804;  
    ch1_dsg_deg <= 16'h0804; 
    ch1_h_blank <= 16'h0010; 
//
    ch2_control <= 16'h3C40; //8'b0100_0000;  
    ch2_lsg_leg <= 16'h0804;  
    ch2_dsg_deg <= 16'h0804;  
    ch2_h_blank <= 16'h0010; 	 
//
    ch3_control <= 16'h3C60; //8'b0110_0000;  
    ch3_lsg_leg <= 16'h0804;  
    ch3_dsg_deg <= 16'h0804;  
    ch3_h_blank <= 16'h0010; 	 
//
    ch0_low_mark  <= 16'h0000;
    ch0_high_mark <= 16'h03FF;
    ch1_low_mark  <= 16'h0000;
    ch1_high_mark <= 16'h03FF;
    ch2_low_mark  <= 16'h0000;
    ch2_high_mark <= 16'h03FF;
    ch3_low_mark  <= 16'h0000;
    ch3_high_mark <= 16'h03FF;	
	 //
    ch0_start_stream <= {8'h00,8'h00,8'hFC,8'h9B,8'h01,8'hE0,8'h01,8'h80,8'h02};
    ch0_stop_stream  <= 8'h00;
    ch1_start_stream <= {8'h00,8'h00,8'hFC,8'h9B,8'h01,8'hE0,8'h01,8'h80,8'h02};
    ch1_stop_stream  <= 8'h00;
    ch2_start_stream <= {8'h00,8'h00,8'hFC,8'h9B,8'h01,8'hE0,8'h01,8'h80,8'h02};
    ch2_stop_stream  <= 8'h00;
    ch3_start_stream <= {8'h00,8'h00,8'hFC,8'h9B,8'h01,8'hE0,8'h01,8'h80,8'h02};
    ch3_stop_stream  <= 8'h0;	 
  end 
  else if (reg_wr)
  begin
    if (reg_adr ==  8'h00) mode_select <= reg_di[7:0];
    if (reg_adr ==  8'h01) power_status <= reg_di[7:0];
	 //
    if (reg_adr ==  8'h10) ch0_control <= reg_di[15:0];
    if (reg_adr ==  8'h11) ch0_lsg_leg <= reg_di[15:0];
    if (reg_adr ==  8'h12) ch0_dsg_deg <= reg_di[15:0];
    if (reg_adr ==  8'h13) ch0_h_blank <= reg_di[15:0];
    //  
    if (reg_adr ==  8'h14) ch1_control <= reg_di[15:0];
    if (reg_adr ==  8'h15) ch1_lsg_leg <= reg_di[15:0];
    if (reg_adr ==  8'h16) ch1_dsg_deg <= reg_di[15:0];
    if (reg_adr ==  8'h17) ch1_h_blank <= reg_di[15:0];	 
    //
    if (reg_adr ==  8'h18) ch2_control <= reg_di[15:0];
    if (reg_adr ==  8'h19) ch2_lsg_leg <= reg_di[15:0];
    if (reg_adr ==  8'h1A) ch2_dsg_deg <= reg_di[15:0];
    if (reg_adr ==  8'h1B) ch2_h_blank <= reg_di[15:0];	 
    //  
    if (reg_adr ==  8'h1C) ch3_control <= reg_di[15:0];
    if (reg_adr ==  8'h1D) ch3_lsg_leg <= reg_di[15:0];
    if (reg_adr ==  8'h1E) ch3_dsg_deg <= reg_di[15:0];
    if (reg_adr ==  8'h1F) ch3_h_blank <= reg_di[15:0];	 
    //
    if (reg_adr ==  8'h20) ch0_low_mark  <= reg_di[15:0];
    if (reg_adr ==  8'h21) ch0_high_mark <= reg_di[15:0];
    if (reg_adr ==  8'h22) ch1_low_mark  <= reg_di[15:0];
    if (reg_adr ==  8'h23) ch1_high_mark <= reg_di[15:0];
    if (reg_adr ==  8'h24) ch2_low_mark  <= reg_di[15:0];
    if (reg_adr ==  8'h25) ch2_high_mark <= reg_di[15:0];
    if (reg_adr ==  8'h26) ch3_low_mark  <= reg_di[15:0];
    if (reg_adr ==  8'h27) ch3_high_mark <= reg_di[15:0];
    //
    if (reg_adr ==  8'h60) ch0_start_stream <= reg_di[71:0];
    if (reg_adr ==  8'h61) ch0_stop_stream  <= reg_di[7:0];
    //
    if (reg_adr ==  8'h90) ch1_start_stream <= reg_di[71:0];
    if (reg_adr ==  8'h91) ch1_stop_stream  <= reg_di[7:0];
    //
    if (reg_adr ==  8'hC0) ch2_start_stream <= reg_di[71:0];
    if (reg_adr ==  8'hC1) ch2_stop_stream  <= reg_di[7:0];
    //
    if (reg_adr ==  8'hF0) ch3_start_stream <= reg_di[71:0];
    if (reg_adr ==  8'hF1) ch3_stop_stream  <= reg_di[7:0];
  end
end
// 
reg  ftdi_sync_ch0_drop;
reg  ftdi_sync_ch1_drop;
reg  ftdi_sync_ch2_drop;
reg  ftdi_sync_ch3_drop;
reg  [1:0] ch0_drop;
reg  [1:0] ch1_drop;
reg  [1:0] ch2_drop;
reg  [1:0] ch3_drop;
// 
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
  begin
    ftdi_sync_ch0_drop <= 'b0;
    ftdi_sync_ch1_drop <= 'b0;
    ftdi_sync_ch2_drop <= 'b0;
    ftdi_sync_ch3_drop <= 'b0;
    ch0_drop <= 'b0;
    ch1_drop <= 'b0;
    ch2_drop <= 'b0;
    ch3_drop <= 'b0;
  end 
  else 
  begin 
    ftdi_sync_ch0_drop <= ch0_drop_trig;
    ftdi_sync_ch1_drop <= ch1_drop_trig;
    ftdi_sync_ch2_drop <= ch2_drop_trig;
    ftdi_sync_ch3_drop <= ch3_drop_trig;
    ch0_drop <= {ch0_drop[0],ftdi_sync_ch0_drop}; 
    ch1_drop <= {ch1_drop[0],ftdi_sync_ch1_drop}; 
    ch2_drop <= {ch2_drop[0],ftdi_sync_ch2_drop}; 
    ch3_drop <= {ch2_drop[0],ftdi_sync_ch3_drop}; 
  end
end 
//
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
    ch0_frame_drop <= 'b0;
  else if (reg_wr & (reg_adr ==  6'h28))
    ch0_frame_drop <= reg_di[31:0];
  else if (ch0_drop[0] & (!ch0_drop[1]))
    ch0_frame_drop <= ch0_frame_drop + 1'b1;
end
// 
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
    ch1_frame_drop <= 'b0;
  else if (reg_wr & (reg_adr ==  6'h29))
    ch1_frame_drop <= reg_di[31:0];
  else if (ch1_drop[0] & (!ch1_drop[1]))
    ch1_frame_drop <= ch1_frame_drop + 1'b1;
end
// 
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
    ch2_frame_drop <= 'b0;
  else if (reg_wr & (reg_adr ==  6'h2A))
    ch2_frame_drop <= reg_di[31:0];
  else if (ch2_drop[0] & (!ch2_drop[1]))
    ch2_frame_drop <= ch2_frame_drop + 1'b1;
end
// 
always @ (posedge clk or negedge rst_n)
begin
  if (~rst_n)
    ch3_frame_drop <= 'b0;
  else if (reg_wr & (reg_adr ==  6'h2B))
    ch3_frame_drop <= reg_di[31:0];
  else if (ch3_drop[0] & (!ch2_drop[1]))
    ch3_frame_drop <= ch3_frame_drop + 1'b1;
end
//
wire [3:0] chn_irq;
assign chn_irq[0] = ch0_control[3] & ch0_drop[0] & (!ch0_drop[1]); 
assign chn_irq[1] = ch1_control[3] & ch1_drop[0] & (!ch1_drop[1]); 
assign chn_irq[2] = ch2_control[3] & ch2_drop[0] & (!ch2_drop[1]); 
assign chn_irq[3] = ch3_control[3] & ch3_drop[0] & (!ch3_drop[1]); 
//
assign int_irq = |chn_irq; 
assign int_ena = mode_select[3];
// 
// assign frame_drop[0] = |ch0_frame_drop;
// assign frame_drop[1] = |ch1_frame_drop;
// assign frame_drop[2] = |ch2_frame_drop;
// assign frame_drop[3] = |ch3_frame_drop;
assign frame_drop = {ch3_drop[0],ch2_drop[0],ch1_drop[0],ch0_drop[0]};
assign power_stat = power_status;
assign wakeup = enable;
//
endmodule 
